#ifndef bqw_consmap_hh
#define bqw_consmap_hh

#include <vector>
#include "cons.hh"

enum allocationmode { am_free, am_used, am_bad };
enum favors { favor_none, favor_x, favor_y, favor_both };

class MapAllocItem
{
    allocationmode m;
public:
    Screen::xycoord free, used;
    MapAllocItem() : m(am_free) { }
    inline operator allocationmode () const { return m; }
    inline MapAllocItem &operator= (allocationmode n) { m = n; return *this; }
};

class UsedMap : private vector<MapAllocItem>
{
    unsigned xs,ys, contentsize;
    unsigned top;

    inline unsigned calcofs(unsigned x, unsigned y) const { return x + (y-top) * xs; }
    
    /*
    void Check(unsigned ofs) const
    {
        if(ofs >= size())
        {
            throw;
        }
    }
    */
    inline MapAllocItem &at(unsigned ofs) { /* Check(ofs); */ return operator[] (ofs); }
    inline const MapAllocItem &at(unsigned ofs) const { /* Check(ofs); */ return operator[] (ofs); }
    
public:
    UsedMap() : xs(0), ys(0), top(0)
    {
    }
    void resize(unsigned x, unsigned y)
    {
        vector<MapAllocItem>::resize(contentsize = (xs=x) * (ys=y));
        top = 0;
    }
    inline unsigned size() const { return contentsize; }
    inline unsigned xsize() const { return xs; }
    inline unsigned ysize() const { return ys + top; }
    inline unsigned ytop() const { return top; }
    
    void SetBox(unsigned x1,unsigned y1, unsigned xsi,unsigned ysi, allocationmode value);

    bool EmptyBox(unsigned x1,unsigned y1, unsigned xsi,unsigned ysi)
    {
        if(xsi < ysi)
        {
            for(unsigned ofs=calcofs(x1,y1), xp=0; xp<xsi; ++xp)
                if(at(ofs++).free.y < ysi)
                    return false;
        }
        else
        {
            for(unsigned ofs=calcofs(x1,y1), yp=0; yp<ysi; ++yp)
            {
                if(at(ofs).free.x < xsi)
                    return false;
                ofs += xs;
            }
        }
        return true;
    }

    // Returns false, if EmptyBox(x1,y1, xmin,ymin) would return false
    bool BiggestEmptyAt(unsigned x1,unsigned y1, Screen::xycoord &dest,
                        unsigned xmin=1, unsigned ymin=1, favors favor=favor_both);

    inline unsigned nanlen_x(unsigned x,unsigned y) const { return at(calcofs(x,y)).used.x; }
    inline unsigned nanlen_y(unsigned x,unsigned y) const { return at(calcofs(x,y)).used.y; }    
    
    void Analyze();
};

#endif
